@@ -37,6 +37,7 @@ import stat
 import sys
 import time
 from multiprocessing import Process
+from shutil import ReadError
 from threading import Thread
 from typing import IO, Any, BinaryIO, Collection, Dict, List, Optional, Tuple, Type, Union
 
@@ -531,6 +532,7 @@ class SevenZipFile(contextlib.AbstractContextManager):
         targets: Optional[Collection[str]] = None,
         return_dict: bool = False,
         callback: Optional[ExtractCallback] = None,
+        recursive: Optional[bool] = False,
     ) -> Optional[Dict[str, IO[Any]]]:
         if callback is None:
             pass
@@ -560,9 +562,14 @@ class SevenZipFile(contextlib.AbstractContextManager):
         fnames: Dict[str, int] = {}  # check duplicated filename in one archive?
         self.q.put(("pre", None, None))
         for f in self.files:
-            if targets is not None and f.filename not in targets:
-                self.worker.register_filelike(f.id, None)
-                continue
+            if targets is not None and recursive is False:
+                if f.filename not in targets:
+                    self.worker.register_filelike(f.id, None)
+                    continue
+            elif targets is not None and recursive is True:
+                if f.filename not in targets and not any([f.filename.startswith(target) for target in targets]):
+                    self.worker.register_filelike(f.id, None)
+                    continue
 
             # When archive has a multiple files which have same name
             # To guarantee order of archive, multi-thread decompression becomes off.
@@ -893,7 +900,7 @@ class SevenZipFile(contextlib.AbstractContextManager):
         f["data"] = bio
         f["filename"] = pathlib.Path(arcname).as_posix()
         f["uncompressed"] = size
-        f["emptystream"] = size == 0
+        f["emptystream"] = False
         f["attributes"] = getattr(stat, "FILE_ATTRIBUTE_ARCHIVE")
         f["creationtime"] = ArchiveTimestamp.from_now()
         f["lastwritetime"] = ArchiveTimestamp.from_now()
@@ -918,6 +925,17 @@ class SevenZipFile(contextlib.AbstractContextManager):
             raise AbsolutePathError(arcname)
         return path
 
+    def _is_none_or_collection(self, t):
+        if t is None:
+            return True
+        if isinstance(t, str):
+            return False
+        if isinstance(t, tuple):
+            return False
+        if isinstance(t, list) or isinstance(t, set):
+            return True
+        return False
+
     # --------------------------------------------------------------------------
     # The public methods which SevenZipFile provides:
     def getnames(self) -> List[str]:
@@ -981,11 +999,17 @@ class SevenZipFile(contextlib.AbstractContextManager):
         self._extract(path=path, return_dict=False, callback=callback)
 
     def read(self, targets: Optional[Collection[str]] = None) -> Optional[Dict[str, IO[Any]]]:
+        if not self._is_none_or_collection(targets):
+            raise TypeError("Wrong argument type given.")
         self._dict = {}
         return self._extract(path=None, targets=targets, return_dict=True)
 
-    def extract(self, path: Optional[Any] = None, targets: Optional[Collection[str]] = None) -> None:
-        self._extract(path, targets, return_dict=False)
+    def extract(
+        self, path: Optional[Any] = None, targets: Optional[Collection[str]] = None, recursive: Optional[bool] = False
+    ) -> None:
+        if not self._is_none_or_collection(targets):
+            raise TypeError("Wrong argument type given.")
+        self._extract(path, targets, return_dict=False, recursive=recursive)
 
     def reporter(self, callback: ExtractCallback):
         while True:
@@ -1067,7 +1091,7 @@ class SevenZipFile(contextlib.AbstractContextManager):
             size = last - current
         else:
             raise ValueError("Wrong argument passed for argument bio.")
-        if size > 0:
+        if size >= 0:
             folder = self.header.initialize()
             file_info = self._make_file_info_from_name(bio, size, arcname)
             self.header.files_info.files.append(file_info)
@@ -1187,9 +1211,10 @@ def unpack_7zarchive(archive, path, extra=None):
     """
     Function for registering with shutil.register_unpack_format().
     """
-    arc = SevenZipFile(archive)
-    arc.extractall(path)
-    arc.close()
+    if not is_7zfile(archive):
+        raise ReadError(f"{archive} is not a 7zip file.")
+    with SevenZipFile(archive) as arc:
+        arc.extractall(path)
 
 
 def pack_7zarchive(base_name, base_dir, owner=None, group=None, dry_run=None, logger=None):
